@@ -3,6 +3,7 @@
 
 #include "document.h"
 
-void                RB_Analyse_Document( struct RB_Document *arg_document );
+void                RB_Analyse_Document(
+    struct RB_Document *arg_document );
 
 #endif /* ROBODOC_ANALYSER_H */
