@@ -14,7 +14,7 @@
  * AUTHOR 
  *   Frans Slothouber
  *****
- * $Id: directory.c,v 1.32 2004/09/25 18:12:53 gumpu Exp $
+ * $Id: directory.c,v 1.40 2007/05/14 15:41:19 petterik Exp $
  */
 
 #include <assert.h>
@@ -25,7 +25,7 @@
 #include "globals.h"
 #include "roboconfig.h"
 
-#if defined (RB_MSVC) || defined(_XCOMPILE_MINGW)
+#if defined (RB_MSVC) || defined(__MINGW32__)
 
 #  include <io.h>
 #  include <sys/types.h>
@@ -64,10 +64,13 @@ char                content_buffer[RB_CBUFFERSIZE + 1];
 
 
 /* Local functions */
-static int          RB_Is_PathCharacter( int c );
-
+static int          RB_Is_PathCharacter(
+    int c );
 
 
+#if defined (RB_MSVC) || defined(__MINGW32__)
+/* empty */
+#else
 
 /****f* Directory/RB_FileType
  * FUNCTION
@@ -77,6 +80,12 @@ static int          RB_Is_PathCharacter( int c );
  *   If _DIRENT_HAVE_D_TYPE is defined we can find the filetype
  *   directly in the a_direntry.  If not we have to stat the file to
  *   find out.
+ * SYNOPSIS
+ */
+T_RB_FileType RB_FileType(
+    char *arg_pathname,
+    struct dirent *a_direntry )
+/*
  * INPUTS
  *   o arg_pathname -- the path leading up to this file
  *   o a_direntry -- a directory entry read by readdir()
@@ -84,12 +93,6 @@ static int          RB_Is_PathCharacter( int c );
  *   The type of the file.
  ******
  */
-
-#if defined (RB_MSVC) || defined(_XCOMPILE_MINGW)
-/* empty */
-#else
-T_RB_FileType
-RB_FileType( char *arg_pathname, struct dirent *a_direntry )
 {
     T_RB_FileType       file_type = RB_FT_UNKNOWN;
 
@@ -149,36 +152,38 @@ RB_FileType( char *arg_pathname, struct dirent *a_direntry )
 }
 #endif
 
-/*x**f* Directory/RB_Directory_Insert_RB_Path
- * NAME
- *   RB_Directory_Insert_RB_Path -- insert a RB_Path into a RB_Directory
- * FUNCTION
- *  The RB_Path is added at the beginning of the list.
- *****
+/****f* Directory/RB_Directory_Insert_RB_Path
+ * SYNOPSIS
  */
 
-void
-RB_Directory_Insert_RB_Path( struct RB_Directory *arg_rb_directory,
-                             struct RB_Path *arg_rb_path )
+void RB_Directory_Insert_RB_Path(
+    struct RB_Directory *arg_rb_directory,
+    struct RB_Path *arg_rb_path )
+/*
+ * FUNCTION
+ *  Insert a RB_Path into a RB_Directory.
+ *  The RB_Path is added at the beginning of the list.
+ * SOURCE
+ */
 {
     arg_rb_path->next = arg_rb_directory->first_path;
     arg_rb_directory->first_path = arg_rb_path;
 }
 
+/*****/
 
-/*x**f* Directory/RB_Directory_Insert_RB_File
- * NAME
- *   RB_Directory_Insert_RB_File -- insert a RB_File into a RB_Directory
+/****f* Directory/RB_Directory_Insert_RB_File
+ * SYNOPSIS
+ */
+void RB_Directory_Insert_RB_Filename(
+    struct RB_Directory *arg_rb_directory,
+    struct RB_Filename *arg_rb_filename )
+/*
  * FUNCTION
  *   Insert an RB_File structure into a RB_Directory structure.
  *   The RB_File is added at the end of the list.
- *
- *****
+ * SOURCE
  */
-
-void
-RB_Directory_Insert_RB_Filename( struct RB_Directory *arg_rb_directory,
-                                 struct RB_Filename *arg_rb_filename )
 {
     if ( arg_rb_directory->last == 0 )
     {
@@ -194,16 +199,20 @@ RB_Directory_Insert_RB_Filename( struct RB_Directory *arg_rb_directory,
     }
 }
 
+/******/
+
 /****f* Directory/RB_Free_RB_Directory
  * FUNCTION
  *   Free all the memory use by the RB_Directory structure.
+ * SYNOPSIS
+ */
+void RB_Free_RB_Directory(
+    struct RB_Directory *arg_directory )
+/*
  * INPUTS
  *   o arg_directory -- the thing to be freed.
  * SOURCE
  */
-
-void
-RB_Free_RB_Directory( struct RB_Directory *arg_directory )
 {
     struct RB_Filename *rb_filename;
     struct RB_Filename *rb_filename2;
@@ -243,6 +252,12 @@ RB_Free_RB_Directory( struct RB_Directory *arg_directory )
  *   specified by the path.
  *   This structure can then be uses to walk through all the
  *   files in the directory and it's subdirectories.
+ * SYNOPSIS
+ */
+struct RB_Directory *RB_Get_RB_Directory(
+    char *arg_rootpath_name,
+    char *arg_docroot_name )
+/*
  * INPUTS
  *   arg_rootpath -- the name a the directory to get,
  *                   a nul terminated string.
@@ -254,12 +269,9 @@ RB_Free_RB_Directory( struct RB_Directory *arg_directory )
  *   A freshly allocated RB_Directory filled with source files.
  * SOURCE
  */
-
-struct RB_Directory *
-RB_Get_RB_Directory( char *arg_rootpath_name, char* arg_docroot_name )
 {
     struct RB_Directory *rb_directory;
-    struct RB_Path* doc_path = NULL;
+    struct RB_Path     *doc_path = NULL;
 
     rb_directory =
         ( struct RB_Directory * ) malloc( sizeof( struct RB_Directory ) );
@@ -274,9 +286,12 @@ RB_Get_RB_Directory( char *arg_rootpath_name, char* arg_docroot_name )
 
     RB_Fill_Directory( rb_directory, rb_directory->first_path, doc_path );
     if ( ( RB_Number_Of_Filenames( rb_directory ) > 0 ) &&
-         ( RB_Number_Of_Paths( rb_directory ) > 0 ) ) {
+         ( RB_Number_Of_Paths( rb_directory ) > 0 ) )
+    {
         RB_SortDirectory( rb_directory );
-    } else {
+    }
+    else
+    {
 
         RB_Panic( "No files found! (Or all were filtered out)\n" );
     }
@@ -290,6 +305,11 @@ RB_Get_RB_Directory( char *arg_rootpath_name, char* arg_docroot_name )
 /****f* Directory/RB_Get_RB_SingleFileDirectory
  * NAME
  *   RB_Get_RB_SingleFileDirectory -- get a RB_Directory structure
+ * SYNOPSIS
+ */
+struct RB_Directory *RB_Get_RB_SingleFileDirectory(
+    char *arg_fullpath )
+/*
  * FUNCTION
  *   Returns a RB_Directory structure to the give directory,
  *   specified by the path that contains only a single file.
@@ -301,9 +321,6 @@ RB_Get_RB_Directory( char *arg_rootpath_name, char* arg_docroot_name )
  *   a single file.
  * SOURCE
  */
-
-struct RB_Directory *
-RB_Get_RB_SingleFileDirectory( char *arg_fullpath )
 {
     struct RB_Directory *rb_directory;
     struct RB_Path     *path;
@@ -346,6 +363,13 @@ RB_Get_RB_SingleFileDirectory( char *arg_fullpath )
 /****f* Directory/RB_Fill_Directory
  * NAME
  *   RB_Fill_Directory -- fill a RB_Directory structure
+ * SYNOPSIS
+ */
+void RB_Fill_Directory(
+    struct RB_Directory *arg_rb_directory,
+    struct RB_Path *arg_path,
+    struct RB_Path *arg_doc_path )
+/*
  * FUNCTION
  *   Walks through all the files in the directory pointed to
  *   by arg_path and adds all the files to arg_rb_directory.
@@ -359,21 +383,17 @@ RB_Get_RB_SingleFileDirectory( char *arg_fullpath )
  *   subdirectories in arg_path.
  * NOTE
  *   This a is a recursive function.
- *****/
-
-void
-RB_Fill_Directory( struct RB_Directory *arg_rb_directory,
-                   struct RB_Path *arg_path,
-                   struct RB_Path *arg_doc_path )
+ * SOURCE
+ */
 {
 
-#if defined (RB_MSVC) || defined(_XCOMPILE_MINGW)
+#if defined (RB_MSVC) || defined(__MINGW32__)
     struct _finddata_t  c_file;
     long                hFile;
     char               *wildcard = NULL;
     int                 len;
 
-    RB_Say( "Scanning %s\n", arg_path->name );
+    RB_Say( "Scanning %s\n", SAY_INFO, arg_path->name );
     len = strlen( arg_path->name ) + strlen( "*.*" ) + 1;
     wildcard = ( char * ) malloc( len );
     assert( wildcard );
@@ -402,32 +422,37 @@ RB_Fill_Directory( struct RB_Directory *arg_rb_directory,
                 }
                 else
                 {
-                    if ( RB_To_Be_Skipped( c_file.name ) ) 
+                    if ( RB_To_Be_Skipped( c_file.name ) )
                     {
                         /* User asked this directory to be skipped */
                     }
                     else
                     {
-                        if ( course_of_action & DO_NODESC )
+						if ( course_of_action.do_nodesc )
                         {
                             /* Don't descent into the subdirectories */
                         }
                         else
                         {
-                            struct RB_Path     *rb_path = RB_Get_RB_Path2( arg_path->name, c_file.name ); 
+                            struct RB_Path     *rb_path =
+                                RB_Get_RB_Path2( arg_path->name,
+                                                 c_file.name );
 
-                            if ( ( arg_doc_path && strcmp( rb_path->name, arg_doc_path->name ) ) ||
-                                    !arg_doc_path
-                               )
+                            if ( ( arg_doc_path
+                                   && strcmp( rb_path->name,
+                                              arg_doc_path->name ) )
+                                 || !arg_doc_path )
                             {
                                 rb_path->parent = arg_path;
                                 RB_Directory_Insert_RB_Path( arg_rb_directory,
-                                        rb_path );
-                                RB_Fill_Directory( arg_rb_directory, rb_path, arg_doc_path );
+                                                             rb_path );
+                                RB_Fill_Directory( arg_rb_directory, rb_path,
+                                                   arg_doc_path );
                             }
                             else
                             {
-                                RB_Say( "skipping %s\n", rb_path->name );
+                                RB_Say( "skipping %s\n", SAY_INFO,
+                                        rb_path->name );
                             }
                         }
                     }
@@ -457,7 +482,7 @@ RB_Fill_Directory( struct RB_Directory *arg_rb_directory,
     struct dirent      *a_direntry;
     DIR                *a_dirstream;
 
-    RB_Say( "Scanning %s\n", arg_path->name );
+    RB_Say( "Scanning %s\n", SAY_INFO, arg_path->name );
     a_dirstream = opendir( arg_path->name );
 
     if ( a_dirstream )
@@ -494,33 +519,37 @@ RB_Fill_Directory( struct RB_Directory *arg_rb_directory,
                 }
                 else
                 {
-                    if ( RB_To_Be_Skipped( a_direntry->d_name ) ) 
+                    if ( RB_To_Be_Skipped( a_direntry->d_name ) )
                     {
                         /* User asked this directory to be skipped */
                     }
                     else
                     {
-                        if ( course_of_action & DO_NODESC )
+                        if ( course_of_action.do_nodesc )
                         {
                             /* Don't descent into the subdirectories */
                         }
                         else
                         {
-                            struct RB_Path     *rb_path = RB_Get_RB_Path2( arg_path->name,
-                                    a_direntry->d_name );
+                            struct RB_Path     *rb_path =
+                                RB_Get_RB_Path2( arg_path->name,
+                                                 a_direntry->d_name );
 
                             rb_path->parent = arg_path;
-                            if ( ( arg_doc_path && strcmp( rb_path->name, arg_doc_path->name ) ) ||
-                                    !arg_doc_path
-                               )
+                            if ( ( arg_doc_path
+                                   && strcmp( rb_path->name,
+                                              arg_doc_path->name ) )
+                                 || !arg_doc_path )
                             {
                                 RB_Directory_Insert_RB_Path( arg_rb_directory,
-                                        rb_path );
-                                RB_Fill_Directory( arg_rb_directory, rb_path, arg_doc_path );
+                                                             rb_path );
+                                RB_Fill_Directory( arg_rb_directory, rb_path,
+                                                   arg_doc_path );
                             }
                             else
                             {
-                                RB_Say( "skipping %s\n", rb_path->name );
+                                RB_Say( "skipping %s\n", SAY_INFO,
+                                        rb_path->name );
                             }
                         }
                     }
@@ -536,10 +565,17 @@ RB_Fill_Directory( struct RB_Directory *arg_rb_directory,
 #endif
 }
 
+/*****/
 
 /****f* Directory/RB_Is_Source_File
  * NAME
  *   RB_Is_Source_File -- Is a file a sourcefile?
+ * SYNOPSIS
+ */
+int RB_Is_Source_File(
+    struct RB_Path *path,
+    char *filename )
+/*
  * FUNCTION
  *   This functions examines the content of a file to
  *   see whether or not it is a sourcefile.
@@ -548,9 +584,6 @@ RB_Fill_Directory( struct RB_Directory *arg_rb_directory,
  *   in the first 8191 characters of the file.
  * SOURCE
  */
-
-int
-RB_Is_Source_File( struct RB_Path *path, char *filename )
 {
     int                 is_source = 1;
 
@@ -561,7 +594,7 @@ RB_Is_Source_File( struct RB_Path *path, char *filename )
     }
     else
     {
-        unsigned int       size = 0;
+        unsigned int        size = 0;
 
         /* Compute the length of the filename including
            the path. */
@@ -630,21 +663,22 @@ RB_Is_Source_File( struct RB_Path *path, char *filename )
  *
  *   This test is done based on the wildcard expressions specified
  *   in configuration.ignore_files.
- *
+ * SYNOPSIS
+ */
+int RB_To_Be_Skipped(
+    char *filename )
+/*
  * INPUTS
  *   o filename -- the name of the file
  * SOURCE
  */
-
-int
-RB_To_Be_Skipped( char *filename )
 {
     unsigned int        i;
     int                 skip = FALSE;
 
     for ( i = 0; i < configuration.ignore_files.number; ++i )
     {
-        if ( RB_Match( filename, configuration.ignore_files.names[ i ] ) )
+        if ( RB_Match( filename, configuration.ignore_files.names[i] ) )
         {
             skip = TRUE;
             break;
@@ -663,27 +697,30 @@ RB_To_Be_Skipped( char *filename )
  *
  *   This test is done based on the wildcard expressions specified
  *   in configuration.accept_files.
- *
+ * SYNOPSIS
+ */
+int RB_Not_Accepted(
+    char *filename )
+/*
  * INPUTS
  *   o filename -- the name of the file
  * SOURCE
  */
-
-int
-RB_Not_Accepted( char *filename )
 {
     unsigned int        i;
     int                 skip = FALSE;
 
-    skip = RB_To_Be_Skipped(filename);
+    skip = RB_To_Be_Skipped( filename );
 
-    if(!skip && configuration.accept_files.number > 0) {
+    if ( !skip && configuration.accept_files.number > 0 )
+    {
         skip = TRUE;
         for ( i = 0; i < configuration.accept_files.number; ++i )
         {
-            if ( RB_Match( filename, configuration.accept_files.names[ i ] ) )
+            if ( RB_Match( filename, configuration.accept_files.names[i] ) )
             {
-                RB_Say( "accept >%s< with >%s<\n", filename, configuration.accept_files.names[ i ] );
+                RB_Say( "accept >%s< with >%s<\n", SAY_INFO, filename,
+                        configuration.accept_files.names[i] );
                 skip = FALSE;
                 break;
             }
@@ -698,6 +735,11 @@ RB_Not_Accepted( char *filename )
 /****f* Directory/RB_Get_FileName
  * NAME
  *   RB_Get_PathName -- extract the file name 
+ * SYNOPSIS
+ */
+char               *RB_Get_FileName(
+    char *arg_fullpath )
+/*
  * FUNCTION
  *   Given a full path to a file, that is a filename, or a filename
  *   prefixed with a pathname, return the filename.
@@ -715,9 +757,6 @@ RB_Not_Accepted( char *filename )
  *   You are responsible for deallocating it.
  * SOURCE
  */
-
-char               *
-RB_Get_FileName( char *arg_fullpath )
 {
     int                 n;
     int                 i;
@@ -773,6 +812,11 @@ RB_Get_FileName( char *arg_fullpath )
 /****f* Directory/RB_Get_PathName
  * NAME
  *   RB_Get_PathName -- extract the path name 
+ * SYNOPSIS
+ */
+char               *RB_Get_PathName(
+    char *arg_fullpath )
+/*
  * FUNCTION
  *   Given a full path to a file, that is a filename, or a filename
  *   prefixed with a pathname, return the pathname.
@@ -790,9 +834,6 @@ RB_Get_FileName( char *arg_fullpath )
  *   You are responsible for deallocating it.
  * SOURCE
  */
-
-char               *
-RB_Get_PathName( char *arg_fullpath )
 {
     int                 n;
     int                 i;
@@ -844,6 +885,11 @@ RB_Get_PathName( char *arg_fullpath )
  *   Test if a character is part of the group of
  *   characters that you would normally find in 
  *   a path.
+ * SYNOPSIS
+ */
+static int RB_Is_PathCharacter(
+    int c )
+/*
  * INPUTS
  *   c -- the character to be tested.
  * RESULT
@@ -851,9 +897,6 @@ RB_Get_PathName( char *arg_fullpath )
  *   FALSE -- it is not.
  * SOURCE
  */
-
-static int
-RB_Is_PathCharacter( int c )
 {
     return ( ( c == ':' ) || ( c == '/' ) );
 }
@@ -863,15 +906,16 @@ RB_Is_PathCharacter( int c )
 
 /* Sort the files and paths */
 
-unsigned int RB_Number_Of_Filenames( struct RB_Directory *arg_rb_directory )
+/* TODO FS Document */
+unsigned int RB_Number_Of_Filenames(
+    struct RB_Directory *arg_rb_directory )
 {
-    unsigned int number_of_filenames = 0;
-    unsigned int i = 0;
+    unsigned int        number_of_filenames = 0;
     struct RB_Filename *rb_filename = NULL;
 
     for ( rb_filename = arg_rb_directory->first;
           rb_filename;
-          ++number_of_filenames, rb_filename = rb_filename->next ) 
+          ++number_of_filenames, rb_filename = rb_filename->next )
     {
         /* Empty */
     }
@@ -879,90 +923,101 @@ unsigned int RB_Number_Of_Filenames( struct RB_Directory *arg_rb_directory )
 }
 
 
-unsigned int RB_Number_Of_Paths( struct RB_Directory *arg_rb_directory )
+/* TODO FS Document */
+unsigned int RB_Number_Of_Paths(
+    struct RB_Directory *arg_rb_directory )
 {
-    unsigned int number_of_paths = 0;
-    unsigned int i = 0;
-    struct RB_Path *rb_path = NULL;
+    unsigned int        number_of_paths = 0;
+    struct RB_Path     *rb_path = NULL;
 
     for ( rb_path = arg_rb_directory->first_path;
-          rb_path;
-          ++number_of_paths, rb_path = rb_path->next ) 
+          rb_path; ++number_of_paths, rb_path = rb_path->next )
     {
         /* Empty */
     }
     return number_of_paths;
 }
 
-int RB_Path_Compare( void* p1, void* p2 )
+/* TODO FS Document */
+int RB_Path_Compare(
+    void *p1,
+    void *p2 )
 {
-    struct RB_Path *path_1 = p1;
-    struct RB_Path *path_2 = p2;
+    struct RB_Path     *path_1 = p1;
+    struct RB_Path     *path_2 = p2;
+
     return RB_Str_Case_Cmp( path_1->name, path_2->name );
 }
 
-int RB_Filename_Compare( void* p1, void* p2 )
+/* TODO FS Document */
+int RB_Filename_Compare(
+    void *p1,
+    void *p2 )
 {
     struct RB_Filename *filename_1 = p1;
     struct RB_Filename *filename_2 = p2;
+
     return RB_Str_Case_Cmp( filename_1->name, filename_2->name );
 }
 
 
-void RB_SortDirectory( struct RB_Directory *arg_rb_directory )
+/* TODO FS Document */
+void RB_SortDirectory(
+    struct RB_Directory *arg_rb_directory )
 {
-    unsigned int number_of_filenames = RB_Number_Of_Filenames( arg_rb_directory );
-    unsigned int number_of_paths     = RB_Number_Of_Paths( arg_rb_directory );
-    unsigned int i;
-    struct RB_Path *rb_path = NULL;
+    unsigned int        number_of_filenames =
+        RB_Number_Of_Filenames( arg_rb_directory );
+    unsigned int        number_of_paths =
+        RB_Number_Of_Paths( arg_rb_directory );
+    unsigned int        i;
+    struct RB_Path     *rb_path = NULL;
     struct RB_Filename *rb_filename = NULL;
-    struct RB_Path** paths = NULL;
-    struct RB_Filename** filenames = NULL;
+    struct RB_Path    **paths = NULL;
+    struct RB_Filename **filenames = NULL;
 
     assert( number_of_filenames > 0 );
     assert( number_of_paths > 0 );
-    paths = calloc( number_of_paths, sizeof( struct RB_Path* ) );
-    filenames = calloc( number_of_filenames, sizeof( struct RB_Filename* ) );
+    paths = calloc( number_of_paths, sizeof( struct RB_Path * ) );
+    filenames = calloc( number_of_filenames, sizeof( struct RB_Filename * ) );
 
 
-    RB_Say( "Sorting Directory\n" );
-    for( i = 0, rb_path = arg_rb_directory->first_path; 
-         rb_path;
-         rb_path = rb_path->next ) 
+    RB_Say( "Sorting Directory\n", SAY_INFO );
+    for ( i = 0, rb_path = arg_rb_directory->first_path;
+          rb_path; rb_path = rb_path->next )
     {
         assert( i < number_of_paths );
-        paths[ i ] = rb_path;
+        paths[i] = rb_path;
         ++i;
     }
 
-   for( i = 0, rb_filename = arg_rb_directory->first; 
-         rb_filename;
-         rb_filename = rb_filename->next ) 
+    for ( i = 0, rb_filename = arg_rb_directory->first;
+          rb_filename; rb_filename = rb_filename->next )
     {
         assert( i < number_of_filenames );
-        filenames[ i ] = rb_filename;
+        filenames[i] = rb_filename;
         ++i;
     }
 
-    RB_QuickSort( paths, 0, number_of_paths - 1, RB_Path_Compare );
-    RB_QuickSort( filenames, 0, number_of_filenames - 1, RB_Filename_Compare );
+    RB_QuickSort( ( void ** ) paths, 0, number_of_paths - 1,
+                  RB_Path_Compare );
+    RB_QuickSort( ( void ** ) filenames, 0, number_of_filenames - 1,
+                  RB_Filename_Compare );
 
-    for( i = 0; i < number_of_paths - 1; ++i )
+    for ( i = 0; i < number_of_paths - 1; ++i )
     {
-        paths[ i ]->next = paths[ i + 1 ];
+        paths[i]->next = paths[i + 1];
     }
-    paths[ number_of_paths - 1 ]->next = NULL;
-    arg_rb_directory->first_path = paths[ 0 ];
+    paths[number_of_paths - 1]->next = NULL;
+    arg_rb_directory->first_path = paths[0];
 
-    for( i = 0; i < number_of_filenames - 1; ++i )
+    for ( i = 0; i < number_of_filenames - 1; ++i )
     {
-        filenames[ i ]->next = filenames[ i + 1 ];
+        filenames[i]->next = filenames[i + 1];
     }
-    filenames[ number_of_filenames - 1 ]->next = NULL;
-    arg_rb_directory->first = filenames[ 0 ];
-    arg_rb_directory->last  = filenames[ number_of_filenames - 1 ];
+    filenames[number_of_filenames - 1]->next = NULL;
+    arg_rb_directory->first = filenames[0];
+    arg_rb_directory->last = filenames[number_of_filenames - 1];
 
     free( paths );
     free( filenames );
 }
-
