@@ -23,13 +23,9 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
-#if defined (RB_MSVC)
-#  include <direct.h>
+#if defined (RB_BCC) || defined( RB_MSVC )
 #else
-#  if defined (RB_BCC)
-#  else
-#  include <unistd.h> /* Not used for Borland */
-#  endif
+#include <unistd.h>           /* Not used for Borland */
 #endif
 
 #include "robodoc.h"
@@ -43,6 +39,7 @@
 #include <string.h>
 #include "generator.h"
 #include "file.h"
+#include "globals.h"
 
 
 #ifdef DMALLOC
@@ -53,14 +50,17 @@
 /****f* Document/RB_Document_Add_Part
  * FUNCTION
  *   Add a new part to the document.
+ * SYNOPSIS
+ */
+void RB_Document_Add_Part(
+    struct RB_Document *document,
+    struct RB_Part *part )
+/*
  * INPUTS
  *   o document  -- the document the part is to be added to.
  *   o part      -- the part to be added
  * SOURCE
  */
-
-void
-RB_Document_Add_Part( struct RB_Document *document, struct RB_Part *part )
 {
     part->next = document->parts;
     document->parts = part;
@@ -70,8 +70,8 @@ RB_Document_Add_Part( struct RB_Document *document, struct RB_Part *part )
 
 
 /* TODO Documentation */
-void
-RB_Free_RB_Document( struct RB_Document *document )
+void RB_Free_RB_Document_Parts(
+    struct RB_Document *document )
 {
     if ( document->parts )
     {
@@ -84,9 +84,18 @@ RB_Free_RB_Document( struct RB_Document *document )
             RB_Free_RB_Part( a_part );
         }
     }
+    document->parts = NULL;
+}
+
+
+/* TODO Documentation */
+void RB_Free_RB_Document(
+    struct RB_Document *document )
+{
+    RB_Free_RB_Document_Parts( document );
     if ( document->headers )
     {
-        unsigned long     i;
+        unsigned long       i;
 
         for ( i = 0; i < document->no_headers; ++i )
         {
@@ -108,8 +117,8 @@ RB_Free_RB_Document( struct RB_Document *document )
  * SOURCE
  */
 
-void
-RB_Document_Create_Parts( struct RB_Document *document )
+void RB_Document_Create_Parts(
+    struct RB_Document *document )
 {
     struct RB_Filename *i_file = NULL;
 
@@ -135,15 +144,16 @@ RB_Document_Create_Parts( struct RB_Document *document )
  *   part or the singledoc name.   The file_name tells in which file
  *   the documentation for the header is to be stored.
  * SYNOPSIS
- *   void RB_Fill_Header_Filename ( struct RB_Document* document )
+ */
+void RB_Fill_Header_Filename(
+    struct RB_Document *document )
+/*
  * SOURCE
  */
-
-void
-RB_Fill_Header_Filename( struct RB_Document *document )
 {
     struct RB_Part     *i_part;
 
+    RB_Say( "Computing file_name attribute for all headers.\n", SAY_DEBUG );
     for ( i_part = document->parts; i_part; i_part = i_part->next )
     {
         struct RB_header   *i_header;
@@ -151,15 +161,15 @@ RB_Fill_Header_Filename( struct RB_Document *document )
         for ( i_header = i_part->headers;
               i_header; i_header = i_header->next )
         {
-            if ( document->actions & DO_SINGLEDOC )
+            if ( document->actions.do_singledoc )
             {
                 i_header->file_name = document->singledoc_name;
             }
-            else if ( document->actions & DO_MULTIDOC )
+            else if ( document->actions.do_multidoc )
             {
                 i_header->file_name = RB_Get_FullDocname( i_part->filename );
             }
-            else if ( document->actions & DO_SINGLEFILE )
+            else if ( document->actions.do_singlefile )
             {
                 i_header->file_name = document->singledoc_name;
             }
@@ -180,7 +190,10 @@ RB_Fill_Header_Filename( struct RB_Document *document )
  *   the path of the source file and the documentation root path and
  *   the source root path.
  * SYNOPSIS
- *   void RB_Document_Determine_DocFilePaths( struct RB_Document* document )
+ */
+void RB_Document_Determine_DocFilePaths(
+    struct RB_Document *document )
+/*
  * EXAMPLE
  *   srcpath = ./test/mysrc/sub1/sub2
  *   srcroot = ./test/mysrc/
@@ -189,9 +202,6 @@ RB_Fill_Header_Filename( struct RB_Document *document )
  *   docpath = ./test/mydoc/sub1/sub2
  * SOURCE
  */
-
-void
-RB_Document_Determine_DocFilePaths( struct RB_Document *document )
 {
     struct RB_Path     *path;
     int                 docroot_length;
@@ -216,10 +226,17 @@ RB_Document_Determine_DocFilePaths( struct RB_Document *document )
         assert( length >= srcroot_length );
         tail = name + srcroot_length;
         new_name = calloc( docroot_length +
-                ( length - srcroot_length ) + 1, sizeof( char ) );
+                           ( length - srcroot_length ) + 1, sizeof( char ) );
         assert( new_name );
         strcat( new_name, document->docroot->name );
-        strcat( new_name, tail );
+        if ( document->actions.do_no_subdirectories )
+        {
+            /* No subdirectory */
+        }
+        else
+        {
+            strcat( new_name, tail );
+        }
         path->docname = new_name;
     }
 }
@@ -233,14 +250,14 @@ RB_Document_Determine_DocFilePaths( struct RB_Document *document )
  *   tree.  It tests if the directories exist and if they
  *   do not the directory is created.
  * SYNOPSIS
- *   void RB_Document_Create_DocFilePaths( struct RB_Document* document )
+ */
+void RB_Document_Create_DocFilePaths(
+    struct RB_Document *document )
+/*
  * INPUTS
  *   o document -- the document for which the tree is created.
  * SOURCE
  */
-
-void
-RB_Document_Create_DocFilePaths( struct RB_Document *document )
 {
     struct RB_Path     *path;
 
@@ -249,7 +266,7 @@ RB_Document_Create_DocFilePaths( struct RB_Document *document )
         char               *pathname = NULL;
         char               *c2 = NULL;
 
-        RB_Say( "Trying to create directory %s\n", path->docname );
+        RB_Say( "Trying to create directory %s\n", SAY_INFO, path->docname );
         /* Don't want to modify the docname in the path
            structure. So we make a copy that we later
            destroy. */
@@ -270,8 +287,8 @@ RB_Document_Create_DocFilePaths( struct RB_Document *document )
                     /* Path exists. */
                 }
                 else if ( ( strlen( pathname ) == 2 ) &&
-                          ( isalpha( pathname[0] ) ) &&
-                          ( pathname[1] == ':' ) ) 
+                          ( utf8_isalpha( pathname[0] ) ) &&
+                          ( pathname[1] == ':' ) )
                 {
                     /* Is is a drive indicator, ( A: B: C: etc )
                      * stat fails on this, but we should not
@@ -282,7 +299,7 @@ RB_Document_Create_DocFilePaths( struct RB_Document *document )
                 {
                     int                 result;
 
-#if defined(RB_MSVC) || defined(_XCOMPILE_MINGW) || defined(__MINGW32_VERSION)
+#if defined(__MINGW32__) || defined( RB_MSVC )
                     result = mkdir( pathname );
 #else
                     result = mkdir( pathname, 0770 );
@@ -308,15 +325,6 @@ RB_Document_Create_DocFilePaths( struct RB_Document *document )
 /*******/
 
 
-/* TODO Documentation, to be replaced with RB_SortCompareHeaderTypes */
-int header_cmp( void* l1, void* l2 )
-{
-    struct RB_header* header_1 = l1;
-    struct RB_header* header_2 = l2;
-    return RB_Str_Case_Cmp( header_1->name, header_2->name );
-}
-
-
 /* TODO Documentation */
 
 /*x**f* 
@@ -329,76 +337,95 @@ int header_cmp( void* l1, void* l2 )
  * SOURCE
  */
 
-int RB_CompareHeaders( void* h1, void* h2 )
+int RB_CompareHeaders(
+    void *h1,
+    void *h2 )
 {
-    struct RB_header* header_1 = h1;
-    struct RB_header* header_2 = h2;
-    if ( ( header_1->htype->typeCharacter == 'h' ) &&
-         ( header_2->htype->typeCharacter != 'h' ) )
+    struct RB_header   *header_1 = h1;
+    struct RB_header   *header_2 = h2;
+
+    // Check for priorities
+    if ( header_1->htype->priority > header_2->htype->priority )
     {
+        // Header 1 has higher priority
         return -1;
     }
-    else if ( ( header_1->htype->typeCharacter != 'h' ) &&
-              ( header_2->htype->typeCharacter == 'h' ) )
+    else if ( header_1->htype->priority < header_2->htype->priority )
     {
+        // Header 2 has higher priority
         return 1;
     }
-    else if ( ( header_1->htype->typeCharacter == 'h' ) &&
-              ( header_2->htype->typeCharacter == 'h' ) )
-    {
-        return RB_Str_Case_Cmp( header_1->name, header_2->name );
-    }
     else
     {
-        return RB_Str_Case_Cmp( header_1->name, header_2->name );
+        // Priorities are equal
+        // Check if we sort on full name or just the function name
+        if ( course_of_action.do_sectionnameonly )
+        {
+            // Do not include parent name in sorting if they are not displayed
+            return RB_Str_Case_Cmp( header_1->function_name,
+                                    header_2->function_name );
+        }
+        else
+        {
+            // Sort on full name ( module/function )
+            return RB_Str_Case_Cmp( header_1->name, header_2->name );
+        }
     }
 }
 
 /*****/
 
 /* TODO Documentation */
-void RB_Document_Sort_Headers( struct RB_Document *document )
+void RB_Document_Sort_Headers(
+    struct RB_Document *document )
 {
-    struct RB_Part*     i_part;
-    struct RB_header**  headers;   /* Pointer to an array of pointers RB_headers. */
+    struct RB_Part     *i_part;
     unsigned long       part_count = 0;
 
-    RB_Say( "Sorting headers per part (file)\n" );
+    RB_Say( "Sorting headers per part (file)\n", SAY_INFO );
     for ( i_part = document->parts; i_part; i_part = i_part->next )
     {
         struct RB_header   *i_header;
+
         /* Count the number of headers */
         for ( part_count = 0, i_header = i_part->headers;
-              i_header; i_header = i_header->next ) { part_count++; }
+              i_header; i_header = i_header->next )
+        {
+            part_count++;
+        }
 
         if ( part_count )
         {
             /* Sort them */
-            struct RB_header**  temp_headers = calloc( part_count, sizeof( struct RB_header* ) );
-            unsigned int i = 0;
+            struct RB_header  **temp_headers =
+                calloc( part_count, sizeof( struct RB_header * ) );
+            unsigned int        i = 0;
+
             i_header = i_part->headers;
             for ( i = 0; i < part_count; ++i )
             {
                 assert( i_header );
-                temp_headers[ i ] = i_header;
+                temp_headers[i] = i_header;
                 i_header = i_header->next;
             }
-            RB_QuickSort( temp_headers, 0, part_count - 1, RB_CompareHeaders );
-            i_part->headers = temp_headers[ 0 ];
+            RB_QuickSort( ( void ** ) temp_headers, 0, part_count - 1,
+                          RB_CompareHeaders );
+            i_part->headers = temp_headers[0];
             i_part->headers->next = NULL;
-            i_header = temp_headers[ 0 ];
+            i_header = temp_headers[0];
             for ( i = 1; i < part_count; ++i )
             {
                 assert( i_header );
-                i_header->next = temp_headers[ i ];
+                i_header->next = temp_headers[i];
                 i_header = i_header->next;
             }
-            temp_headers[ part_count - 1 ]->next = NULL;
+            temp_headers[part_count - 1]->next = NULL;
             free( temp_headers );
         }
     }
-    RB_Say( "Sorting all headers\n");
-    RB_QuickSort( document->headers, 0, document->no_headers - 1, RB_CompareHeaders );
+    RB_Say( "Sorting all headers\n", SAY_INFO );
+    RB_QuickSort( ( void ** ) document->headers, 0, document->no_headers - 1,
+                  RB_CompareHeaders );
 }
 
 
@@ -415,36 +442,39 @@ void RB_Document_Sort_Headers( struct RB_Document *document )
  * SOURCE
  */
 
-void
-RB_Document_Collect_Headers( struct RB_Document *document )
+void RB_Document_Collect_Headers(
+    struct RB_Document *document )
 {
-    struct RB_Part*     i_part;
-    struct RB_header**  headers;   /* Pointer to an array of pointers RB_headers. */
+    struct RB_Part     *i_part;
+    struct RB_header  **headers;        /* Pointer to an array of pointers RB_headers. */
     unsigned long       count = 0;
     unsigned long       part_count = 0;
     unsigned long       i = 0;
 
-    RB_Say( "Collecting all headers in a single table\n" );
+    RB_Say( "Collecting all headers in a single table\n", SAY_INFO );
     for ( i_part = document->parts; i_part; i_part = i_part->next )
     {
         struct RB_header   *i_header;
+
         /* Count the number of headers */
         for ( part_count = 0, i_header = i_part->headers;
-              i_header; i_header = i_header->next ) { part_count++; }
+              i_header; i_header = i_header->next )
+        {
+            part_count++;
+        }
         /* Compute the total count */
         count += part_count;
     }
     headers =
-        ( struct RB_header** ) calloc( count, sizeof( struct RB_header* ) );
+        ( struct RB_header ** ) calloc( count, sizeof( struct RB_header * ) );
     for ( i_part = document->parts; i_part; i_part = i_part->next )
     {
         struct RB_header   *i_header;
 
         for ( i_header = i_part->headers;
-              i_header; 
-              i_header = i_header->next )
+              i_header; i_header = i_header->next )
         {
-            headers[ i ] = i_header;
+            headers[i] = i_header;
             i++;
         }
     }
@@ -456,20 +486,27 @@ RB_Document_Collect_Headers( struct RB_Document *document )
 
 /* TODO Documentation */
 
-struct RB_header* RB_Document_Check_For_Duplicate( struct RB_Document* arg_document, char* arg_name )
+struct RB_header   *RB_Document_Check_For_Duplicate(
+    struct RB_Document *arg_document,
+    struct RB_header *hdr )
 {
-    struct RB_Part*     i_part;
+    struct RB_Part     *i_part;
+
     for ( i_part = arg_document->parts; i_part; i_part = i_part->next )
     {
         struct RB_header   *i_header;
 
-        for ( i_header = i_part->headers;
-              i_header; i_header = i_header->next )
+        for ( i_header = i_part->headers; i_header;
+              i_header = i_header->next )
         {
-            if( strcmp( i_header->name, arg_name ) == 0 )
-            {
-                return i_header;
-            }
+            int                 i;
+
+            if ( hdr == i_header )
+                continue;
+
+            for ( i = 0; i < hdr->no_names; i++ )
+                if ( strcmp( hdr->names[i], i_header->name ) == 0 )
+                    return i_header;
         }
     }
     return NULL;
@@ -481,17 +518,17 @@ struct RB_header* RB_Document_Check_For_Duplicate( struct RB_Document* arg_docum
  parent of B
 */
 
-void
-RB_Document_Link_Headers( struct RB_Document *document )
+void RB_Document_Link_Headers(
+    struct RB_Document *document )
 {
-    unsigned long                i;
-    unsigned long                j;
+    unsigned long       i;
+    unsigned long       j;
     struct RB_header   *parent;
     struct RB_header   *child;
     char               *parent_name;
     char               *child_name;
 
-    RB_Say( "Linking headers\n" );
+    RB_Say( "Linking all %d headers\n", SAY_INFO, document->no_headers );
     for ( i = 0; i < document->no_headers; i++ )
     {
         parent = ( document->headers )[i];
@@ -511,17 +548,93 @@ RB_Document_Link_Headers( struct RB_Document *document )
     }
 }
 
+
+void RB_Document_Split_Parts(
+    struct RB_Document *document )
+{
+    struct RB_Part     *i_part = NULL;
+    struct RB_Part    **new_parts = NULL;
+    int                 new_number_of_parts = 0;
+    int                 n = 0;
+    int                 i;
+
+    /* split eacht part in several parts. One for each header
+     * in the original part.
+     */
+    for ( i_part = document->parts; i_part; i_part = i_part->next )
+    {
+        struct RB_header   *i_header;
+
+        for ( i_header = i_part->headers;
+              i_header; i_header = i_header->next )
+        {
+            ++new_number_of_parts;
+        }
+    }
+
+    new_parts = calloc( new_number_of_parts, sizeof( struct RB_Part * ) );
+
+    if ( new_parts )
+    {
+        /* Create new parts */
+
+        RB_Say( "Splitting parts based on headers.\n", SAY_DEBUG );
+        for ( i_part = document->parts; i_part; i_part = i_part->next )
+        {
+            struct RB_header   *i_header;
+            struct RB_header   *i_next_header;
+
+            for ( i_header = i_part->headers;
+                  i_header; i_header = i_next_header )
+            {
+                struct RB_Part     *new_part;
+
+                i_next_header = i_header->next;
+
+                RB_Say( "Creating new part.\n", SAY_DEBUG );
+                new_part = RB_Get_RB_Part(  );
+                RB_Part_Add_Source( new_part,
+                                    RB_Copy_RB_Filename( RB_Part_Get_Source
+                                                         ( i_part ) ) );
+                /* remove header from i_part and add to new_part.
+                 */
+                RB_Part_Add_Header( new_part, i_header );
+                assert( n < new_number_of_parts );
+                new_parts[n] = new_part;
+                ++n;
+            }
+            i_part->headers = NULL;
+            i_part->last_header = NULL;
+        }
+        /* Remove old part from document */
+        RB_Free_RB_Document_Parts( document );
+        /* Add new parts to document */
+        for ( i = 0; i < n; ++i )
+        {
+            RB_Document_Add_Part( document, new_parts[i] );
+        }
+        /* clean-up temp array */
+        free( new_parts );
+    }
+    else
+    {
+        RB_Panic( "Out of memory! RB_Document_Split_Parts()" );
+    }
+}
+
 /* TODO Documentation */
-void
-RB_Document_Determine_DocFileNames( struct RB_Document *document )
+void RB_Document_Determine_DocFileNames(
+    struct RB_Document *document )
 {
     struct RB_Filename *filename;
-    unsigned int       length = 0;
+    unsigned int        length = 0;
     char               *name;
     char               *c;
 
     struct RB_Part     *part;
 
+    assert( document->actions.do_multidoc );
+
     for ( part = document->parts; part; part = part->next )
     {
 
@@ -531,6 +644,20 @@ RB_Document_Determine_DocFileNames( struct RB_Document *document )
         length = strlen( filename->name );
         /* add one for the '.' */
         ++length;
+
+        if ( document->actions.do_one_file_per_header )
+        {
+            struct RB_header   *i_header = part->headers;
+
+            assert( i_header );
+            /* add the name of the header to the filename */
+            /* We make this twice as long because some of the
+             * characters in the file are escaped to 2 hexadecimal
+             * digits.
+             */
+            length += 2 * strlen( i_header->name );
+        }
+
         length += RB_Get_Len_Extension( document->extension );
         /* plus one for the '\0' */
         ++length;
@@ -544,7 +671,32 @@ RB_Document_Determine_DocFileNames( struct RB_Document *document )
                 *c = '_';
             }
         }
+
+        if ( document->actions.do_one_file_per_header )
+        {
+            unsigned int        i;
+            struct RB_header   *i_header = part->headers;
+
+            assert( i_header );
+            /* add the name of the header to the filename */
+            for ( i = 0; i < strlen( i_header->name ); ++i )
+            {
+                if ( utf8_isalnum( i_header->name[i] ) )
+                {
+                    sprintf( c, "%c", i_header->name[i] );
+                    c++;
+                }
+                else
+                {
+                    sprintf( c, "%2X", i_header->name[i] );
+                    c++;
+                    c++;
+                }
+            }
+        }
         RB_Add_Extension( document->extension, name );
+
+        RB_Say( "Filename for part is %s\n", SAY_DEBUG, name );
         part->filename->docname = name;
     }
 }
@@ -554,14 +706,14 @@ RB_Document_Determine_DocFileNames( struct RB_Document *document )
  *   Open the file that will contain the documentation in
  *   case we create a single document.
  * SYNOPSIS
- *   FILE* RB_Open_SingleDocumentation( struct RB_Document* document )
+ */
+FILE               *RB_Open_SingleDocumentation(
+    struct RB_Document *document )
+/*
  * RESULT
  *   An opened file.
- *****
+ * SOURCE
  */
-
-FILE               *
-RB_Open_SingleDocumentation( struct RB_Document *document )
 {
     FILE               *file;
     static char        *default_name = "singledoc";
@@ -600,22 +752,24 @@ RB_Open_SingleDocumentation( struct RB_Document *document )
     {
         RB_Panic( "Can't open %s\n", name );
     }
+    free( name );
     return file;
 }
 
+/****/
 
 /****f* Document/RB_Get_RB_Document
  * FUNCTION
  *   Allocate and initialize an RB_Document structure.
  * SYNOPSIS
- *   struct RB_Document* RB_Get_RB_Document()
+ */
+struct RB_Document *RB_Get_RB_Document(
+    void )
+/*
  * RESULT
  *   An initialized document structure.
  * SOURCE
  */
-
-struct RB_Document *
-RB_Get_RB_Document( void )
 {
     struct RB_Document *document = 0;
     document =
@@ -627,7 +781,7 @@ RB_Get_RB_Document( void )
         document->links = NULL;
         document->headers = NULL;
         document->doctype = UNKNOWN;
-        document->actions = 0;
+        document->actions = No_Actions();
         document->srctree = NULL;
         document->srcroot = NULL;
         document->docroot = NULL;
@@ -635,6 +789,9 @@ RB_Get_RB_Document( void )
         document->no_headers = 0;
         document->charset = NULL;
         document->extension = NULL;
+        document->first_section_level = 1;
+        document->doctype_name = NULL;
+        document->doctype_location = NULL;
     }
     else
     {
@@ -644,4 +801,3 @@ RB_Get_RB_Document( void )
 }
 
 /*******/
-
