@@ -34,20 +34,32 @@ struct RB_Filename
 /******/
 
 
-struct RB_Filename *RB_Get_RB_Filename( char *arg_filename,
-                                        struct RB_Path *arg_rb_path );
-void                RB_Free_RB_Filename( struct RB_Filename
-                                         *arg_rb_filename );
-/* */
-char               *Get_Fullname( struct RB_Filename *arg_rb_filename );
-void                RB_Set_FullDocname( struct RB_Filename *arg_rb_filename, char* name );
-char               *RB_Get_FullDocname( struct RB_Filename *arg_rb_filename );
-char               *RB_Get_Path( struct RB_Filename *arg_rb_filename );
-char               *RB_Get_Filename( struct RB_Filename *arg_rb_filename );
-char               *RB_Get_Extension( struct RB_Filename *arg_rb_filename );
+struct RB_Filename *RB_Get_RB_Filename(
+    char *arg_filename,
+    struct RB_Path *arg_rb_path );
+void                RB_Free_RB_Filename(
+    struct RB_Filename *arg_rb_filename );
 
 /* */
-void                RB_Filename_Dump( struct RB_Filename *arg_rb_filename );
+char               *Get_Fullname(
+    struct RB_Filename *arg_rb_filename );
+void                RB_Set_FullDocname(
+    struct RB_Filename *arg_rb_filename,
+    char *name );
+char               *RB_Get_FullDocname(
+    struct RB_Filename *arg_rb_filename );
+char               *RB_Get_Path(
+    struct RB_Filename *arg_rb_filename );
+char               *RB_Get_Filename(
+    struct RB_Filename *arg_rb_filename );
+char               *RB_Get_Extension(
+    struct RB_Filename *arg_rb_filename );
+struct RB_Filename *RB_Copy_RB_Filename(
+    struct RB_Filename *arg_rb_filename );
+
+/* */
+void                RB_Filename_Dump(
+    struct RB_Filename *arg_rb_filename );
 
 
 #endif /* ROBODOC_FILE_H */
