@@ -19,15 +19,16 @@
 #include "roboconfig.h"
 #include "util.h"
 
-#define NO_MARKER_LOCKED -1
-#define NO_MARKER        -2
+/* This limits the total number of possible markers. */
+/* TODO should be an enum */
+#define NO_MARKER_LOCKED 100000
+#define NO_MARKER        100002
 
-static int locked_header_marker = NO_MARKER_LOCKED;
-static int locked_end_marker    = NO_MARKER_LOCKED;
-static int locked_remark_marker = NO_MARKER_LOCKED;
+static int          locked_header_marker = NO_MARKER_LOCKED;
+static int          locked_end_marker = NO_MARKER_LOCKED;
+static int          locked_remark_marker = NO_MARKER_LOCKED;
 
-
-/****v* Headers/header_markers [3.0h]
+/****v* Headers/header_markers
  * NAME
  *   header_markers -- strings that mark the begin of a header.
  * FUNCTION
@@ -72,7 +73,7 @@ char               *header_markers[] = {
 
 /****/
 
-/****v* Headers/robo_header [3.0h]
+/****v* Headers/robo_header
  * NAME
  *   robo_header -- the distinct robodoc header - 
  *                  alternative to using header_markers
@@ -81,11 +82,11 @@ char               *header_markers[] = {
  *   ROBODOC confuses asterisks commonly used in comments as a header.
  *   To use this header instead of header_markers use the -rh switch.
  * NOTE
- *   Added by David Druffner.
+ *   Added by David Druffner.   OBSOLETE
  * SOURCE
  */
 
-char               *robo_header = "/*ROBODOC*";
+char               *robo_header = "/*ROBODOC*"; /* TODO Remove */
 
 /****/
 
@@ -106,10 +107,10 @@ char               *robo_header = "/*ROBODOC*";
 char               *remark_markers[] = {
 
     " *",                       /* 0  C, C++, Pascal, Modula-2 */
-    "//!",                      /* 1  C++, ACM */  /* MUST CHECK BEFORE C++ */
+    "//!",                      /* 1  C++, ACM *//* MUST CHECK BEFORE C++ */
     "//",                       /* 2  C++ */
     "*",                        /* 3  C, C++, M68K assembler, Pascal,  Modula-2 */
-    ";;!",                      /* 4  Aspen Plus */   /* MUST CHECK BEFORE M68K */
+    ";;!",                      /* 4  Aspen Plus *//* MUST CHECK BEFORE M68K */
     ";*",                       /* 5  M68K assembler */
     ";",                        /* 6  M68K assembler */
     "C",                        /* 7  Fortran */
@@ -148,12 +149,12 @@ char               *remark_markers[] = {
 
 char               *end_markers[] = {
     "/***",                     /* 0  C, C++ */
-    "//!***",                   /* 1  C++, ACM */  /* Must check before C++ */
+    "//!***",                   /* 1  C++, ACM *//* Must check before C++ */
     "//***",                    /* 2  C++ */
     " ***",                     /* 3  C, C++, Pascal, Modula-2 */
     "{***",                     /* 4  Pascal */
     "(***",                     /* 5  Pascal, Modula-2, B52 */
-    ";;!***",                   /* 6  Aspen Plus */  /* Must check before M68K */
+    ";;!***",                   /* 6  Aspen Plus *//* Must check before M68K */
     ";***",                     /* 7  M68K assembler */
     "***",                      /* 8  M68K assembler */
     "C     ***",                /* 9  Fortran */
@@ -205,7 +206,7 @@ char               *end_remark_markers[] = {
     "--->",                     /* 16  HTML */
     ( char * ) NULL,            /* 17  GNU Assembler */
     ( char * ) NULL,            /* 18  FORTRAN 90 !! */
-    ( char * ) NULL,            /* 19  FORTRAN 90 !*/
+    ( char * ) NULL,            /* 19  FORTRAN 90 ! */
     ( char * ) NULL,            /* 20  VB */
     ( char * ) NULL,            /* 21  Aspen Plus */
     ( char * ) NULL             /* 22  Forth */
@@ -234,10 +235,14 @@ char               *robo_end[] = { "/*ROBODOC_END*", "*ROBODOC_END*", NULL };
 
 /***f* Headers/RB_Is_Begin_Marker
  * FUNCTION
- *   Scan a line and see if any of the begin of a header markers 
+ *   Scan a line and see if any of the begin-of-a-header-markers 
  *   defined in header_markers can be found.
  * SYNOPSIS
- *   static int RB_Is_Begin_Marker( char* cur_line, char** type )
+ */
+int RB_Is_Begin_Marker(
+    char *cur_line,
+    char **type )
+/*
  * INPUTS
  *   cur_line -- line to be searched.
  * OUTPUT
@@ -247,67 +252,59 @@ char               *robo_end[] = { "/*ROBODOC_END*", "*ROBODOC_END*", NULL };
  *   FALSE -- no begin header was found.
  * SOURCE
  */
-
-int
-RB_Is_Begin_Marker( char *cur_line, char **type )
 {
     int                 found = FALSE;
-    int                 marker = NO_MARKER;  /* for the assert */
+    unsigned int        marker = NO_MARKER;     /* for the assert */
     char               *cur_mchar = NULL;
     char               *cur_char = NULL;
 
-    if ( !( course_of_action & DO_ROBO_HEAD ) 
+    if ( !( course_of_action.do_robo_head )
          &&
-         ( ( ( course_of_action & DO_LOCKHEADER ) &&
-             ( locked_header_marker == NO_MARKER_LOCKED ) 
-           )
-           ||
-           !( course_of_action & DO_LOCKHEADER ) 
-         )
-       )
+         ( ( ( course_of_action.do_lockheader ) &&
+             ( locked_header_marker == NO_MARKER_LOCKED ) )
+           || !( course_of_action.do_lockheader ) ) )
     {
         for ( marker = 0;
               ( marker < configuration.header_markers.number ) && !found;
               marker++ )
         {
-            cur_mchar = configuration.header_markers.names[ marker ];
+            cur_mchar = configuration.header_markers.names[marker];
             for ( found = TRUE, cur_char = RB_Skip_Whitespace( cur_line );
                   *cur_mchar && *cur_char && found; cur_mchar++, cur_char++ )
             {
                 if ( tolower( *cur_mchar ) != tolower( *cur_char ) )
                     found = FALSE;
             }
+            if ( *cur_mchar != '\0' )
+            {
+                /* It is not a complete match */
+                found = FALSE;
+            }
         }
     }
-    else if ( ( course_of_action & DO_LOCKHEADER ) &&
+    else if ( ( course_of_action.do_lockheader ) &&
               ( locked_header_marker != NO_MARKER_LOCKED ) )
     {
-        cur_mchar = configuration.header_markers.names[ locked_header_marker ];
+        cur_mchar = configuration.header_markers.names[locked_header_marker];
         for ( found = TRUE, cur_char = RB_Skip_Whitespace( cur_line );
-                *cur_mchar && *cur_char && found; cur_mchar++, cur_char++ )
+              *cur_mchar && *cur_char && found; cur_mchar++, cur_char++ )
         {
             if ( tolower( *cur_mchar ) != tolower( *cur_char ) )
             {
                 found = FALSE;
             }
+            if ( *cur_mchar != '\0' )
+            {
+                /* It is not a complete match */
+                found = FALSE;
+            }
         }
     }
     else
     {
-        /* Obsolete */
-        /* use robo_header instead if -rh command line switch */
-        if ( ( ( cur_mchar = robo_header ) != NULL ) )
-        {
-            for ( found = TRUE, cur_char = line_buffer;
-                  *cur_mchar && *cur_char && found; cur_mchar++, cur_char++ )
-            {
-                if ( tolower( *cur_mchar ) != tolower( *cur_char ) )
-                {
-                    found = FALSE;
-                }
-            }
-        }
-    }                           /* end of DO_ROBO_HEAD else */
+        assert( 0 );
+    }
+
 
     if ( found && cur_char )
     {
@@ -316,13 +313,13 @@ RB_Is_Begin_Marker( char *cur_line, char **type )
         if ( *cur_char == '*' )
         {
             ++cur_char;
-            found = isspace( *cur_char );
+            found = utf8_isspace( *cur_char );
         }
-        else if ( *(cur_char + 1) == '*') 
+        else if ( *( cur_char + 1 ) == '*' )
         {
             ++cur_char;
             ++cur_char;
-            found = isspace( *cur_char );
+            found = utf8_isspace( *cur_char );
         }
         else
         {
@@ -334,7 +331,7 @@ RB_Is_Begin_Marker( char *cur_line, char **type )
             /* It should contain some non * characters. */
             for ( ; *cur_char; ++cur_char )
             {
-                if ( isalnum( *cur_char )  )
+                if ( utf8_isalnum( *cur_char ) )
                 {
                     found = TRUE;
                 }
@@ -342,13 +339,14 @@ RB_Is_Begin_Marker( char *cur_line, char **type )
         }
     }
 
-    if ( found && 
-         ( course_of_action & DO_LOCKHEADER ) && 
-         ( locked_header_marker == NO_MARKER_LOCKED ) ) 
+    if ( found &&
+         ( course_of_action.do_lockheader ) &&
+         ( locked_header_marker == NO_MARKER_LOCKED ) )
     {
         assert( marker != NO_MARKER );
         locked_header_marker = marker - 1;
-        RB_Say( "header marker locked on %s\n", configuration.header_markers.names[ locked_header_marker ] );
+        RB_Say( "header marker locked on %s\n", SAY_INFO,
+                configuration.header_markers.names[locked_header_marker] );
     }
     return found;
 }
@@ -356,12 +354,126 @@ RB_Is_Begin_Marker( char *cur_line, char **type )
 /******/
 
 
+/* Generic function to skip a remark begin or end marker */
+
+static char        *RB_Skip_Remark_XXX_Marker(
+    char *cur_line,
+    struct Parameters *parameters )
+{
+    char               *cur_char;
+    char               *space_pos;
+    unsigned int        marker;
+    int                 found = FALSE;
+
+    cur_char = RB_Skip_Whitespace( cur_line );
+    space_pos = strchr( cur_char, ' ' );
+    /* Replace the first space on the line with a '\0'
+     * this makes the comparison with the remark markers
+     * much easier.
+     */
+    if ( space_pos )
+    {
+        *space_pos = '\0';
+    }
+
+    for ( marker = 0; ( marker < parameters->number ) && !found; marker++ )
+    {
+        found =
+            ( RB_Str_Case_Cmp( cur_char, parameters->names[marker] ) == 0 );
+    }
+
+    assert( found );
+
+    if ( space_pos )
+    {
+        *space_pos = ' ';
+        return space_pos;
+    }
+    else
+    {
+        return cur_char + strlen( parameters->names[marker - 1] );
+    }
+}
+
+
+/* Generic function to see if there is a remark begin or end marker */
+
+static int RB_Is_Remark_XXX_Marker(
+    char *cur_line,
+    struct Parameters *parameters )
+{
+    char               *cur_char;
+    char               *space_pos;
+    unsigned int        marker;
+    int                 found = FALSE;
+
+    cur_char = RB_Skip_Whitespace( cur_line );
+    space_pos = strchr( cur_char, ' ' );
+    /* Replace the first space on the line with a '\0'
+     * this makes the comparison with the remark markers
+     * much easier.
+     */
+    if ( space_pos )
+    {
+        *space_pos = '\0';
+    }
+
+    for ( marker = 0; ( marker < parameters->number ) && !found; marker++ )
+    {
+        found =
+            ( RB_Str_Case_Cmp( cur_char, parameters->names[marker] ) == 0 );
+    }
+
+    if ( space_pos )
+    {
+        *space_pos = ' ';
+    }
+
+    return found;
+}
+
+
+/* TODO Documentation */
+int RB_Is_Remark_End_Marker(
+    char *cur_line )
+{
+    return RB_Is_Remark_XXX_Marker( cur_line,
+                                    &( configuration.remark_end_markers ) );
+}
+
+/* TODO Documentation */
+int RB_Is_Remark_Begin_Marker(
+    char *cur_line )
+{
+    return RB_Is_Remark_XXX_Marker( cur_line,
+                                    &( configuration.remark_begin_markers ) );
+}
+
+char               *RB_Skip_Remark_Begin_Marker(
+    char *cur_line )
+{
+    return RB_Skip_Remark_XXX_Marker( cur_line,
+                                      &( configuration.
+                                         remark_begin_markers ) );
+}
+
+char               *RB_Skip_Remark_End_Marker(
+    char *cur_line )
+{
+    return RB_Skip_Remark_XXX_Marker( cur_line,
+                                      &( configuration.remark_end_markers ) );
+}
+
+
 /****f* Headers/RB_Is_End_Marker
  * FUNCTION
  *   Scan a line and see if any of the end of a header markers 
  *   defined in header_markers can be found.
  * SYNOPSIS
- *   static int RB_Is_End_Marker( char* cur_line, char** type )
+ */
+int RB_Is_End_Marker(
+    char *cur_line )
+/*
  * INPUTS
  *   cur_line -- line to be searched.
  * OUTPUT
@@ -371,24 +483,17 @@ RB_Is_Begin_Marker( char *cur_line, char **type )
  *   FALSE -- none was found.
  * SOURCE
  */
-
-int
-RB_Is_End_Marker( char *cur_line )
 {
     int                 found = FALSE;
-    int                 marker = NO_MARKER; /* For the assert */
+    unsigned int        marker = NO_MARKER;     /* For the assert */
     char               *cur_mchar;
     char               *cur_char;
 
-    if ( !( course_of_action & DO_ROBO_HEAD ) 
+    if ( !( course_of_action.do_robo_head )
          &&
-         ( ( ( course_of_action & DO_LOCKHEADER ) &&
-             ( locked_end_marker == NO_MARKER_LOCKED ) 
-           )
-           ||
-           !( course_of_action & DO_LOCKHEADER ) 
-         )
-       )
+         ( ( ( course_of_action.do_lockheader ) &&
+             ( locked_end_marker == NO_MARKER_LOCKED ) )
+           || !( course_of_action.do_lockheader ) ) )
     {
         for ( marker = 0;
               ( marker < configuration.end_markers.number ) && !found;
@@ -396,10 +501,11 @@ RB_Is_End_Marker( char *cur_line )
         {
             cur_mchar = configuration.end_markers.names[marker];
             cur_char = RB_Skip_Whitespace( cur_line );
-            if ( *cur_char ) 
+            if ( *cur_char )
             {
                 for ( found = TRUE;
-                        *cur_mchar && *cur_char && found; cur_mchar++, cur_char++ )
+                      *cur_mchar && *cur_char && found;
+                      cur_mchar++, cur_char++ )
                 {
                     if ( tolower( *cur_mchar ) != tolower( *cur_char ) )
                     {
@@ -409,16 +515,15 @@ RB_Is_End_Marker( char *cur_line )
             }
         }
     }
-    else if ( ( course_of_action & DO_LOCKHEADER ) &&
+    else if ( ( course_of_action.do_lockheader ) &&
               ( locked_end_marker != NO_MARKER_LOCKED ) )
     {
-        cur_mchar = configuration.end_markers.names[ locked_end_marker ];
+        cur_mchar = configuration.end_markers.names[locked_end_marker];
         cur_char = RB_Skip_Whitespace( cur_line );
-        if ( *cur_char ) 
+        if ( *cur_char )
         {
             for ( found = TRUE;
-                  *cur_mchar && *cur_char && found; 
-                  cur_mchar++, cur_char++ )
+                  *cur_mchar && *cur_char && found; cur_mchar++, cur_char++ )
             {
                 if ( tolower( *cur_mchar ) != tolower( *cur_char ) )
                 {
@@ -429,36 +534,22 @@ RB_Is_End_Marker( char *cur_line )
     }
     else
     {
-        /* TODO OBSOLETE */
-        /*use robo_end if using -rh switch */
-        for ( marker = 0;
-              ( ( cur_mchar = robo_end[marker] ) != NULL ) && !found;
-              marker++ )
-        {
-            for ( found = TRUE, cur_char = cur_line;
-                  *cur_mchar && *cur_char && found; cur_mchar++, cur_char++ )
-            {
-                if ( tolower( *cur_mchar ) != tolower( *cur_char ) )
-                {
-                    found = FALSE;
-                }
-            }
-        }
+        assert( 0 );
     }
 
     /* Locking on end markers does not work at the moment,
      * because there can be more than one end marker for
-     * a given language.
-     * TODO 
+     * a given language. TODO 
      */
-#if 0    
+#if 0
     if ( found &&
-         ( course_of_action & DO_LOCKHEADER ) && 
-         ( locked_end_marker == NO_MARKER_LOCKED ) ) 
+         ( course_of_action.do_LOCKHEADER ) &&
+         ( locked_end_marker == NO_MARKER_LOCKED ) )
     {
         assert( marker != NO_MARKER );
         locked_end_marker = marker - 1;
-        RB_Say( "end marker locked on %s\n", end_markers[ locked_end_marker ] );
+        RB_Say( "end marker locked on %s\n", SAY_INFO,
+                end_markers[locked_end_marker] );
     }
 #endif
     return found;
@@ -472,6 +563,11 @@ RB_Is_End_Marker( char *cur_line )
  *   Check if a line starts with a remark marker.  This function
  *   assumes that the remark marker starts on the first character of
  *   the line.
+ * SYNOPSIS
+ */
+int RB_Has_Remark_Marker(
+    char *lline_buffer )
+/*
  * INPUTS
  *   o lline_buffer -- the line of text.
  * RESULT
@@ -479,13 +575,11 @@ RB_Is_End_Marker( char *cur_line )
  *   o FALSE -- it does not.
  * SOURCE
  */
-
-int RB_Has_Remark_Marker( char* lline_buffer )
 {
-    int                 marker       = NO_MARKER; 
-    int                 marker_found = NO_MARKER;
-    int                 found  = FALSE;
-    char*               space_pos = NULL;
+    unsigned int        marker = 0;
+    unsigned int        marker_found = configuration.remark_markers.number;
+    int                 found = FALSE;
+    char               *space_pos = NULL;
 
     space_pos = strchr( lline_buffer, ' ' );
 
@@ -498,15 +592,16 @@ int RB_Has_Remark_Marker( char* lline_buffer )
         *space_pos = '\0';
     }
 
-    if ( ( ( course_of_action & DO_LOCKHEADER ) &&
+    if ( ( ( course_of_action.do_lockheader ) &&
            ( locked_remark_marker == NO_MARKER_LOCKED ) )
-            ||
-           !( course_of_action & DO_LOCKHEADER ) 
-       )
+         || !( course_of_action.do_lockheader ) )
     {
-        for ( marker = 0; marker < configuration.remark_markers.number; marker++ )
+        for ( marker = 0; marker < configuration.remark_markers.number;
+              marker++ )
         {
-            if ( RB_Str_Case_Cmp( lline_buffer, configuration.remark_markers.names[ marker ] ) == 0 ) 
+            if ( RB_Str_Case_Cmp
+                 ( lline_buffer,
+                   configuration.remark_markers.names[marker] ) == 0 )
             {
                 marker_found = marker;
                 found = TRUE;
@@ -515,22 +610,24 @@ int RB_Has_Remark_Marker( char* lline_buffer )
     }
     else
     {
-        if ( RB_Str_Case_Cmp( lline_buffer, configuration.remark_markers.names[ locked_remark_marker ] ) == 0 ) 
+        if ( RB_Str_Case_Cmp
+             ( lline_buffer,
+               configuration.remark_markers.names[locked_remark_marker] ) ==
+             0 )
         {
             marker_found = marker;
             found = TRUE;
         }
     }
 
-    if ( found && 
-         ( locked_remark_marker == NO_MARKER_LOCKED ) 
-         &&
-         ( course_of_action & DO_LOCKHEADER )
-       ) 
+    if ( found &&
+         ( locked_remark_marker == NO_MARKER_LOCKED )
+         && ( course_of_action.do_lockheader ) )
     {
-        assert( marker_found != NO_MARKER );
+        assert( marker_found < configuration.remark_markers.number );
         locked_remark_marker = marker_found;
-        RB_Say( "remark marker locked on %s\n", configuration.remark_markers.names[ locked_remark_marker ] );
+        RB_Say( "remark marker locked on %s\n", SAY_INFO,
+                configuration.remark_markers.names[locked_remark_marker] );
     }
 
     /* Restore the space we replaced with a '\0' */
@@ -549,27 +646,26 @@ int RB_Has_Remark_Marker( char* lline_buffer )
  * NAME
  *    RB_Skip_Remark_Marker
  * SYNOPSIS
- *    text  = RB_Skip_Remark_Marker (line_buffer)
- *    char *                            char *
+ */
+char               *RB_Skip_Remark_Marker(
+    char *lline_buffer )
+/*
  * FUNCTION
  *    Scan and search for a recognized remark marker; skip past the
  *    marker to the body of the text
  * SOURCE
  */
-
-char*
-RB_Skip_Remark_Marker( char *lline_buffer )
 {
-    int                 marker, found;
+    unsigned int        marker, found;
     char               *cur_char, *cur_mchar;
 
     found = FALSE;
     cur_char = NULL;
     for ( marker = 0;
-         ( marker < configuration.remark_markers.number ) && !found;
-         marker++ )
+          ( marker < configuration.remark_markers.number ) && !found;
+          marker++ )
     {
-        cur_mchar = configuration.remark_markers.names[ marker ];
+        cur_mchar = configuration.remark_markers.names[marker];
         for ( found = TRUE, cur_char = lline_buffer;
               *cur_mchar && *cur_char && found; cur_mchar++, cur_char++ )
         {
@@ -584,15 +680,19 @@ RB_Skip_Remark_Marker( char *lline_buffer )
 
 /**************/
 
+
+
+
 /* TODO Documentation */
-void RB_Header_Lock_Reset( void )
+void RB_Header_Lock_Reset(
+    void )
 {
     locked_header_marker = NO_MARKER_LOCKED;
-    locked_end_marker    = NO_MARKER_LOCKED;
+    locked_end_marker = NO_MARKER_LOCKED;
 }
 
-void RB_Item_Lock_Reset( void )
+void RB_Item_Lock_Reset(
+    void )
 {
     locked_remark_marker = NO_MARKER_LOCKED;
 }
-
