@@ -1,3 +1,4 @@
+/* vi: ff=unix spell */
 /****h* ROBODoc/Items
  * FUNCTION
  *   This module contains functions that deal with items.  The
@@ -14,6 +15,7 @@
 #include <stddef.h>
 #include <string.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <ctype.h>
 #include <assert.h>
 
@@ -35,7 +37,8 @@ char                item_name_buffer[MAX_ITEM_NAME_LENGTH];
 /*****/
 
 /* TODO Documentation */
-char* RB_Get_Item_Name( void )
+char               *RB_Get_Item_Name(
+    void )
 {
     return item_name_buffer;
 }
@@ -45,9 +48,11 @@ char* RB_Get_Item_Name( void )
  * SOURCE
  */
 
-struct RB_Item* RB_Create_Item( enum ItemType arg_item_type )
+struct RB_Item     *RB_Create_Item(
+    enum ItemType arg_item_type )
 {
-    struct RB_Item* item = malloc( sizeof( struct RB_Item ) );
+    struct RB_Item     *item = malloc( sizeof( struct RB_Item ) );
+
     assert( item );
 
     item->next = 0;
@@ -72,17 +77,20 @@ struct RB_Item* RB_Create_Item( enum ItemType arg_item_type )
  * SOURCE
  */
 
-int
-RB_Get_Item_Type( char *cmp_name )
+int RB_Get_Item_Type(
+    char *cmp_name )
 {
     unsigned int        item_type;
 
     assert( configuration.items.number );
-    for ( item_type = 0; 
-          item_type < configuration.items.number; 
-          ++item_type )
+    for ( item_type = 0; item_type < configuration.items.number; ++item_type )
     {
-        if ( !strcmp( configuration.items.names[item_type], cmp_name ) )
+        char               *item = configuration.items.names[item_type];
+
+        /* Skip preformat mark */
+        if ( *item == '-' )
+            item++;
+        if ( !strcmp( item, cmp_name ) )
         {
             return ( item_type );
         }
@@ -113,27 +121,27 @@ RB_Get_Item_Type( char *cmp_name )
  * SOURCE
  */
 
-enum ItemType
-RB_Is_ItemName( char *line )
+enum ItemType RB_Is_ItemName(
+    char *line )
 {
     char               *cur_char = line;
     int                 i = 0;
 
     cur_char = RB_Skip_Whitespace( cur_char );
-    if ( RB_Has_Remark_Marker( cur_char ) ) 
+    if ( RB_Has_Remark_Marker( cur_char ) )
     {
         cur_char = RB_Skip_Remark_Marker( cur_char );
         cur_char = RB_Skip_Whitespace( cur_char );
         /* It there anything left? */
         if ( strlen( cur_char ) )
         {
-            enum ItemType item_type = NO_ITEM;
+            enum ItemType       item_type = NO_ITEM;
 
             /* Copy the name */
             strcpy( item_name_buffer, cur_char );
             /* remove any trailing spaces */
             for ( i = strlen( item_name_buffer ) - 1;
-                    i >= 0 && isspace( item_name_buffer[i] ); --i )
+                  i >= 0 && utf8_isspace( item_name_buffer[i] ); --i )
             {
                 item_name_buffer[i] = '\0';
             }
@@ -141,15 +149,15 @@ RB_Is_ItemName( char *line )
             if ( strlen( item_name_buffer ) )
             {
                 item_type = RB_Get_Item_Type( item_name_buffer );
-#if 0   /* Until we find a better method */
-                if ( item_type == NO_ITEM ) 
+#if 0                           /* Until we find a better method */
+                if ( item_type == NO_ITEM )
                 {
                     /* Check if it is a misspelled item name */
                     item_type = POSSIBLE_ITEM;
                     for ( i = 0; i < strlen( item_name_buffer ); ++i )
                     {
-                        if ( !( isupper( item_name_buffer[ i ] ) ||
-                                    isspace( item_name_buffer[ i ] ) ) )
+                        if ( !( utf8_isupper( item_name_buffer[i] ) ||
+                                utf8_isspace( item_name_buffer[i] ) ) )
                         {
                             /* No it is not */
                             item_type = NO_ITEM;
@@ -174,9 +182,11 @@ RB_Is_ItemName( char *line )
 
 /******/
 
-int Is_Ignore_Item( char* name )
+/* TODO Documentation */
+int Is_Ignore_Item(
+    char *name )
 {
-    unsigned int       i;
+    unsigned int        i;
 
     for ( i = 0; i < configuration.ignore_items.number; ++i )
     {
@@ -188,3 +198,119 @@ int Is_Ignore_Item( char* name )
     return FALSE;
 }
 
+
+/****f* HeaderTypes/Works_Like_SourceItem
+ * FUNCTION
+ *   Tells wether this item works similar to the
+ *   source item, that is weather it copies it's
+ *   content verbatim to the output document.
+ * SYNPOPSIS
+ */
+int Works_Like_SourceItem(
+    enum ItemType item_type )
+/*
+ * INPUTS
+ *   item_type -- Type of item (also the index to the item name)
+ * RESULT
+ *   TRUE  -- Item works like a SOURCE item
+ *   FALSE -- Item does NOT work like a SOURCE item
+ * SOURCE
+ */
+{
+    unsigned int        i;
+
+    // Check if it is a SOURCE item
+    if ( item_type == SOURCECODE_ITEM )
+    {
+        return TRUE;
+    }
+
+    // Lookup if it works like a SOURCE item
+    for ( i = 0; i < configuration.source_items.number; ++i )
+    {
+        if ( !strcmp
+             ( configuration.source_items.names[i],
+               configuration.items.names[item_type] ) )
+        {
+            return TRUE;
+        }
+    }
+
+    // Neither SOURCE item, nor works like it
+    return FALSE;
+}
+
+/******/
+
+/****f* HeaderTypes/Is_Preformatted_Item
+ * FUNCTION
+ *   Tells wether this item should be automatically preformatted in the
+ *   output.
+ * SYNPOPSIS
+ */
+int Is_Preformatted_Item(
+    enum ItemType item_type )
+/*
+ * INPUTS
+ *   item_type -- Type of item (also the index to the item name)
+ * RESULT
+ *   TRUE  -- Item should be automatically preformatted
+ *   FALSE -- Item should NOT be automatically preformatted
+ * SOURCE
+ */
+{
+    unsigned int        i;
+
+    // Lookup if item should be preformatted
+    for ( i = 0; i < configuration.preformatted_items.number; ++i )
+    {
+        if ( !strcmp
+             ( configuration.preformatted_items.names[i],
+               configuration.items.names[item_type] ) )
+        {
+            // Item name match, it sould be preformatted
+            return TRUE;
+        }
+    }
+
+    // Do not automatically preformat item
+    return FALSE;
+}
+
+/******/
+
+/****f* HeaderTypes/Is_Format_Item
+ * FUNCTION
+ *   Tells wether this item should be formatted by the browser
+ * SYNPOPSIS
+ */
+int Is_Format_Item(
+    enum ItemType item_type )
+/*
+ * INPUTS
+ *   item_type -- Type of item (also the index to the item name)
+ * RESULT
+ *   TRUE  -- Item should be formatted by the browser
+ *   FALSE -- Item should be left alone
+ * SOURCE
+ */
+{
+    unsigned int        i;
+
+    // Lookup if item should be formatted by the browser
+    for ( i = 0; i < configuration.format_items.number; ++i )
+    {
+        if ( !strcmp
+             ( configuration.format_items.names[i],
+               configuration.items.names[item_type] ) )
+        {
+            // Item name match, it sould be formatted by the browser
+            return TRUE;
+        }
+    }
+
+    // Leave item alone
+    return FALSE;
+}
+
+/******/
