@@ -25,18 +25,29 @@ struct RB_link
     char               *label_name;
     char               *object_name;
     char               *file_name;
-    struct RB_HeaderType* htype;
+    struct RB_HeaderType *htype;
     int                 is_internal;
 };
 
 /*********/
 
-int                 Find_Link( char *word_begin, char **object_name,
-                                  char **unique_name, char **file_name );
-void                RB_CollectLinks( struct RB_Document *document, struct RB_header **headers, unsigned long count );
-void                RB_Free_Links( void );
-void                RB_Free_Link( struct RB_link *link );
+int                 Find_Link(
+    char *word_begin,
+    char **object_name,
+    char **unique_name,
+    char **file_name );
+void                RB_CollectLinks(
+    struct RB_Document *document,
+    struct RB_header **headers,
+    unsigned long count );
+void                RB_Free_Links(
+    void );
+void                RB_Free_Link(
+    struct RB_link *arg_link );
 
-int RB_Number_Of_Links( struct RB_HeaderType* header_type, char* file_name, int internal );
+int                 RB_Number_Of_Links(
+    struct RB_HeaderType *header_type,
+    char *file_name,
+    int internal );
 
 #endif /* ROBODOC_LINKS_H */
