@@ -23,9 +23,12 @@ struct RB_Path
 
 /*****/
 
-struct RB_Path     *RB_Get_RB_Path( char *arg_pathname );
-struct RB_Path     *RB_Get_RB_Path2( char *arg_current_path,
-                                     char *arg_subdirectory );
-void                RB_Free_RB_Path( struct RB_Path *arg_rb_path );
+struct RB_Path     *RB_Get_RB_Path(
+    char *arg_pathname );
+struct RB_Path     *RB_Get_RB_Path2(
+    char *arg_current_path,
+    char *arg_subdirectory );
+void                RB_Free_RB_Path(
+    struct RB_Path *arg_rb_path );
 
 #endif /* ROBODOC_PATH_H */
