@@ -1,4 +1,4 @@
-/* $Id: robohdrs.c,v 1.12 2003/12/28 20:10:42 gumpu Exp $ */
+/* $Id: robohdrs.c,v 1.13 2005/03/02 21:06:41 gumpu Exp $ */
 
 /****h* Docuwala/ROBOhdrs
 *  NAME
@@ -34,9 +34,30 @@
 *******
 */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+
+
+
+#ifndef HAVE_FORK
+
+int main( int argc, char** argv )
+{
+    printf( "Not supported on this platform\n" );
+    printf( "This program requires fork()\n" );
+    return EXIT_SUCCESS;
+}
+
+#else
+
+
+
 #include <assert.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -873,3 +894,7 @@ main( int argc, char **argv )
 }
 
 /********** main */
+
+
+#endif /*  HAVE_FORK  */
+
