@@ -7,14 +7,15 @@
 #include "util.h"
 
 
-char               *
-RB_XMLDB_Get_Default_Extension( void )
+char               *RB_XMLDB_Get_Default_Extension(
+    void )
 {
     return ".xml";
 }
 
-void
-RB_XMLDB_Generate_String( FILE * dest_doc, char *a_string )
+void RB_XMLDB_Generate_String(
+    FILE *dest_doc,
+    char *a_string )
 {
     int                 i;
     int                 l = strlen( a_string );
@@ -29,8 +30,9 @@ RB_XMLDB_Generate_String( FILE * dest_doc, char *a_string )
 
 /* TODO Documentation */
 
-void
-RB_XMLDB_Generate_Label( FILE* dest_doc, char *name )
+void RB_XMLDB_Generate_Label(
+    FILE *dest_doc,
+    char *name )
 {
     int                 i;
     int                 l = strlen( name );
@@ -40,14 +42,15 @@ RB_XMLDB_Generate_Label( FILE* dest_doc, char *name )
     for ( i = 0; i < l; ++i )
     {
         c = name[i];
-        if ( isalnum( c) ) 
+        if ( utf8_isalnum( c ) )
         {
             RB_XMLDB_Generate_Char( dest_doc, c );
         }
         else
         {
-            char buf[4];
-            sprintf(buf, "%02x", c );
+            char                buf[4];
+
+            sprintf( buf, "%02x", c );
             RB_XMLDB_Generate_Char( dest_doc, buf[0] );
             RB_XMLDB_Generate_Char( dest_doc, buf[1] );
         }
@@ -66,8 +69,9 @@ RB_XMLDB_Generate_Label( FILE* dest_doc, char *name )
  * SOURCE
  */
 
-void
-RB_XMLDB_Generate_Char( FILE * dest_doc, int c )
+void RB_XMLDB_Generate_Char(
+    FILE *dest_doc,
+    int c )
 {
     switch ( c )
     {
@@ -87,65 +91,87 @@ RB_XMLDB_Generate_Char( FILE * dest_doc, int c )
         fprintf( dest_doc, "&amp;" );
         break;
     default:
-        RB_FputcLatin1ToUtf8( dest_doc, c );
-        break;
+        // All others are printed literally
+        fputc( c, dest_doc );
     }
 }
 
 /*****/
 
-void
-RB_XMLDB_Generate_Header_Start( FILE * dest_doc,
-                                struct RB_header *cur_header )
+void RB_XMLDB_Generate_Header_Start(
+    FILE *dest_doc,
+    struct RB_header *cur_header )
 {
     fprintf( dest_doc, "<section>\n" );
-/*    fprintf( dest_doc, "<section id=\"%s\">\n", cur_header->unique_name ); */
+    /*    fprintf( dest_doc, "<section id=\"%s\">\n", cur_header->unique_name ); */
     fprintf( dest_doc, "<title>\n" );
     RB_XMLDB_Generate_String( dest_doc, cur_header->name );
     fprintf( dest_doc, "</title>\n" );
 }
 
-void
-RB_XMLDB_Generate_Header_End( FILE * dest_doc, struct RB_header *cur_header )
+void RB_XMLDB_Generate_Header_End(
+    FILE *dest_doc,
+    struct RB_header *cur_header )
 {
+    USE( cur_header );
     fprintf( dest_doc, "</section>\n" );
 }
 
-void
-RB_XMLDB_Generate_Link( FILE * dest, char *dest_name, char *filename,
-                        char *labelname, char *linkname, int in_fold )
+void RB_XMLDB_Generate_Link(
+    FILE *dest,
+    char *dest_name,
+    char *filename,
+    char *labelname,
+    char *linkname )
 {
+    USE( dest_name );
+    USE( filename );
+
     fprintf( dest, "<link linkend=\"%s\">", labelname );
     RB_XMLDB_Generate_String( dest, linkname );
     fprintf( dest, "</link>" );
 }
 
-void
-RB_XMLDB_Generate_Doc_Start( FILE * dest_doc, char *src_name, char *name,
-                             char toc )
+void RB_XMLDB_Generate_Doc_Start(
+    struct RB_Document *document,
+    FILE *dest_doc,
+    char *charset )
 {
-
-    if ( course_of_action & DO_HEADLESS ) 
+    if ( course_of_action.do_headless )
     {
         /* The user does not want the document head. */
     }
     else
     {
-        fprintf( dest_doc, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
-        fprintf( dest_doc, "%s",
-                "<!DOCTYPE article PUBLIC \"-//OASIS//DTD DocBook XML V4.2//EN\" \n"
-                "\"http://www.oasis-open.org/docbook/xml/4.2/docbookx.dtd\">\n"
-                "<article lang=\"en\">\n" );
-        fprintf( dest_doc, "%s",
-                "<articleinfo> <title>API Documentation</title> </articleinfo>\n" );
+        fprintf( dest_doc, "<?xml version=\"1.0\" encoding=\"%s\"?>\n",
+                 charset ? charset : DEFAULT_CHARSET );
+        if ( document->doctype_name && document->doctype_location )
+        {
+            fprintf( dest_doc, "<!DOCTYPE article PUBLIC \"%s\"\n\"%s\">\n",
+                     document->doctype_name, document->doctype_location );
+            fprintf( dest_doc, "%s", "<article lang=\"en\">\n" );
+        }
+        else
+        {
+            fprintf( dest_doc, "%s",
+                     "<!DOCTYPE article PUBLIC \"-//OASIS//DTD DocBook XML V4.2//EN\" \n"
+                     "\"http://www.oasis-open.org/docbook/xml/4.2/docbookx.dtd\">\n"
+                     "<article lang=\"en\">\n" );
+        }
+        fprintf( dest_doc,
+                 "<articleinfo>\n  <title>%s</title>\n</articleinfo>\n",
+                 document_title ? document_title : DEFAULT_DOCTITILE );
     }
 }
 
 
-void
-RB_XMLDB_Generate_Doc_End( FILE * dest_doc, char *name )
+void RB_XMLDB_Generate_Doc_End(
+    FILE *dest_doc,
+    char *name )
 {
-    if ( course_of_action & DO_FOOTLESS ) 
+    USE( name );
+
+    if ( course_of_action.do_footless )
     {
         /* The user does not want the foot of the
          * document.
@@ -158,92 +184,109 @@ RB_XMLDB_Generate_Doc_End( FILE * dest_doc, char *name )
 }
 
 
-void
-RB_XMLDB_Generate_Item_Name( FILE * dest_doc, char *name )
+void RB_XMLDB_Generate_Item_Name(
+    FILE *dest_doc,
+    char *name )
 {
-
     fprintf( dest_doc, "<formalpara><title>" );
     RB_XMLDB_Generate_String( dest_doc, name );
     fprintf( dest_doc, "</title><para></para></formalpara>\n" );
 }
 
-void
-RB_XMLDB_Generate_Item_Begin( FILE* dest_doc )
+void RB_XMLDB_Generate_Item_Begin(
+    FILE *dest_doc )
 {
-
+    USE( dest_doc );
+    /* Empty */
 }
 
 
-void
-RB_XMLDB_Generate_Item_End( FILE * dest_doc )
+void RB_XMLDB_Generate_Item_End(
+    FILE *dest_doc )
 {
-
+    USE( dest_doc );
+    /* Empty */
 }
 
 
-void
-RB_XMLDB_Generate_BeginSection( FILE * dest_doc, int depth, char *name )
+void RB_XMLDB_Generate_BeginSection(
+    FILE *dest_doc,
+    int depth,
+    char *name )
 {
+    USE( depth );
+
     fprintf( dest_doc, "%s", "<section>\n<title>" );
     RB_XMLDB_Generate_String( dest_doc, name );
     fprintf( dest_doc, "%s", "</title>\n" );
 }
 
-void
-RB_XMLDB_Generate_EndSection( FILE * dest_doc, int depth, char *name )
+void RB_XMLDB_Generate_EndSection(
+    FILE *dest_doc,
+    int depth,
+    char *name )
 {
+    USE( depth );
+    USE( name );
     fprintf( dest_doc, "%s", "</section>\n" );
 }
 
 
-void
-RB_XMLDB_Generate_False_Link( FILE * dest_doc, char *name )
+void RB_XMLDB_Generate_False_Link(
+    FILE *dest_doc,
+    char *name )
 {
     RB_XMLDB_Generate_String( dest_doc, name );
 }
 
 
 
-void XMLDB_Generate_Begin_Paragraph( FILE* dest_doc )
+void XMLDB_Generate_Begin_Paragraph(
+    FILE *dest_doc )
 {
     fprintf( dest_doc, "%s", "<para>\n" );
 }
 
-void XMLDB_Generate_End_Paragraph( FILE* dest_doc )
+void XMLDB_Generate_End_Paragraph(
+    FILE *dest_doc )
 {
     fprintf( dest_doc, "%s", "</para>\n" );
 }
 
 
-void XMLDB_Generate_Begin_Preformatted( FILE* dest_doc )
+void XMLDB_Generate_Begin_Preformatted(
+    FILE *dest_doc )
 {
     fprintf( dest_doc, "%s", "<literallayout class=\"monospaced\">\n" );
 }
 
-void XMLDB_Generate_End_Preformatted( FILE* dest_doc )
+void XMLDB_Generate_End_Preformatted(
+    FILE *dest_doc )
 {
     fprintf( dest_doc, "%s", "</literallayout>\n" );
 }
 
 
-void XMLDB_Generate_Begin_List( FILE* dest_doc )
+void XMLDB_Generate_Begin_List(
+    FILE *dest_doc )
 {
     fprintf( dest_doc, "%s", "<itemizedlist>" );
 }
 
-void XMLDB_Generate_End_List( FILE* dest_doc )
+void XMLDB_Generate_End_List(
+    FILE *dest_doc )
 {
     fprintf( dest_doc, "%s", "</itemizedlist>" );
 }
 
-void XMLDB_Generate_Begin_List_Item( FILE* dest_doc )
+void XMLDB_Generate_Begin_List_Item(
+    FILE *dest_doc )
 {
     fprintf( dest_doc, "%s", "<listitem><para>" );
 }
 
-void XMLDB_Generate_End_List_Item( FILE* dest_doc )
+void XMLDB_Generate_End_List_Item(
+    FILE *dest_doc )
 {
     fprintf( dest_doc, "%s", "</para></listitem>" );
 }
-
-
